@@ -4,25 +4,28 @@ use strict;
 use warnings;
 use utf8;
 
-use Carp;
 use DateTime;
 use NorisOtrsReport::OTRS_API;
+use Carp;
+use Data::Dump qw(pp);
 
 use base 'Exporter';
 our $VERSION = "0.1";
 our @EXPORT_OK = qw(generate run_fixture_conversions load_fixture);
 our %EXPORT_TAGS = (
-    data => [
-        qw(data_otrs_ticket data_customtext data_customdate data_workedtime
-          make_data_articles data_state_history data_linked_tickets)
-    ],
-    convert => [qw(convert_epoch_to_date_hash convert_isodate_to_date_hash)],
+    data => 
+        [qw(data_otrs_ticket data_customtext data_customdate 
+            make_data_articles data_state_history data_linked_tickets
+        )],
+    convert => 
+        [qw(convert_epoch_to_date_hash convert_isodate_to_date_hash)],
 );
 Exporter::export_ok_tags('data');
 Exporter::export_ok_tags('convert');
 
 =head1 NAME
 
+
 NorisOtrsReport::Framework - Reporting-Framework für OTRS
 
 =head1 SYNOPSIS
@@ -34,29 +37,29 @@ NorisOtrsReport::Framework - Reporting-Framework für OTRS
  use NorisOtrsReport::DebugProcessor;
 
  generate(
-     UserID      => 1,
-     Search      => { TicketNumber => 10000902 },
-     Conversions => [
-         [ CreateTimeUnix => \&convert_epoch_to_date_hash, 'created' ],
-         [ RealTillTimeNotUsed => \&convert_epoch_to_date_hash, 'pending-until' ],
-         [ CreateTime => \&convert_isodate_to_date_hash, 'created' ],
-     ],
-     Collectors => [
-         \&data_otrs_ticket,
-         \&data_customtext,
-         \&data_customdate,
-         \&data_workedtime,
-         \&data_state_history,
-         \&data_linked_tickets,
-         make_data_articles(
-             qw(note-external-error note-external-reason
-               note-external-actions note-external-opcall
-               note-external-progression ),
-         ),
-     ],
-     LoadFixture => 'filename.json',
-     Processors  => [ NorisOtrsReport::DebugProcessor->new() ],
- );
+        UserID => 1,
+        Search => { TicketNumber => 10000902 },
+        Conversions => 
+            [
+                [ CreateTimeUnix => \&convert_epoch_to_date_hash, 'created' ],
+                [ RealTillTimeNotUsed => \&convert_epoch_to_date_hash, 'pending-until' ],
+                [ CreateTime  => \&convert_isodate_to_date_hash, 'created' ],
+            ],
+        Collectors => [
+                \&data_otrs_ticket,
+                \&data_customtext, 
+                \&data_customdate,
+                \&data_state_history,
+                \&data_linked_tickets,
+                make_data_articles(
+                        qw(note-external-error note-external-reason 
+                        note-external-actions note-external-opcall 
+                        note-external-progression ),
+                    ),
+            ],
+        LoadFixture => 'filename.json',
+        Processors => [NorisOtrsReport::DebugProcessor->new()],
+    );
 
 =for example end
 
@@ -116,27 +119,26 @@ Funktion eingebunden.
 # Verwendung der "Freitextfelder" in otrs:
 # Name => Nummer
 my %TICKET_FREE_TEXT_KEYS = (
-    'Info'                => 1,
-    'Planzeit (Std.)'     => 2,
-    'Sicherheitsrelevant' => 3,
-    'Freigabestatus'      => 4,
-    'Knowledge-Base'      => 5,
-    'Externe ID'          => 6,
-    'AppendKey'           => 12,
-    'SLA-Relevanz'        => 13,
-    'Verantwortlich'      => 14,
-    'Störungsart'         => 15,
-    'Störungsursache'     => 16,
-);
+        'Info' => 1,
+        'externe Servicestelle' => 4,
+        'Externe-ID' => 6,
+        'Mandant' => 8,
+        'Kategorie/Anwendung' => 9,
+        'Störung nach Change' => 10,
+        'Prio QC/Primondo' => 11,
+        'Op-Call' => 12,
+        'Standort/MA' => 13,
+        'Business-Impact' => 14,
+    );
 
 # Verwendung der Freizeitfelder in otrs:
 # Name => Nummer
 my %TICKET_FREE_DATE_KEYS = (
-    'Due-Date'             => 1,
-    'Incident-Start'       => 2,
-    'Incident-Resolved'    => 3,
-    'Dienstleister-Termin' => 4,
-);
+        'Due-Date' => 1,
+        'Incident-Start' => 2,
+        'Incident-Resolved' => 3,
+        'Dienstleister-Termin' => 4,
+    );
 
 =head1 SUBROUTINES
 
@@ -237,48 +239,45 @@ mit den konvertierten Daten
 
 sub generate {
     my %Param = @_;
-
-    my ( $Search, $Collectors );
-    my $Conversions = $Param{Conversions} || {};
+    
+    my ($Search, $Collectors);
+    my $Conversions = $Param{Conversions} || {};    
     my $LoadFixture = $Param{LoadFixture} || '';
     my $FixtureData;
     if ($LoadFixture) {
-        die "generate: Die Parameter 'LoadFixture' und 'Collectors' sind inkompatibel!"
-          if ( $Param{Collectors} );
-        die "generate: Die Parameter 'LoadFixture' und 'Search' sind inkompatibel!"
-          if ( $Param{Search} );
-        $Search      = [1];
+        die "generate: Die Parameter 'LoadFixture' und 'Collectors' sind inkompatibel!" if ($Param{Collectors});
+        die "generate: Die Parameter 'LoadFixture' und 'Search' sind inkompatibel!" if ($Param{Search});
+        $Search = [ 1 ];
         $FixtureData = load_fixture($LoadFixture);
         my $FixtureIndex = 0;
-        $Collectors = [
+        $Collectors = [ 
             sub {
-                my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
-                %$Data = %{ $FixtureData->[ $FixtureIndex++ ] };
-                run_fixture_conversions( $Conversions, $Data );
+                my ($UserID, $Conversions, $TicketID, $Data) = @_;
+                %$Data = %{$FixtureData->[$FixtureIndex++]};
+                run_fixture_conversions($Conversions, $Data);
                 return $Data;
-              }
+            }
         ];
     }
     else {
-        $Search     = $Param{Search}     || die "Missing Parameter: Search";
-        $Collectors = $Param{Collectors} || [ \&data_otrs_ticket ];
+        $Search = $Param{Search} || die "Missing Parameter: Search";
+        $Collectors = $Param{Collectors} || [\&data_otrs_ticket];
     }
-
     my $UserID = $Param{UserID} || 1;
-
-    my $Processors = $Param{Processors} || [ NorisOtrsReport::DebugProcessor->new() ];
-
+    
+    my $Processors = $Param{Processors} || [NorisOtrsReport::DebugProcessor->new()];
+    
     for my $Processor (@$Processors) {
         $Processor->start();
     }
-
+    
     for my $CurrentSearch (@$Search) {
         for my $Processor (@$Processors) {
             $Processor->start_search();
         }
         my @TicketIDs;
         if ($LoadFixture) {
-            @TicketIDs = map $_->{TicketID}, @$FixtureData;
+            @TicketIDs = map $_->{TicketID}, @$FixtureData
         }
         else {
             @TicketIDs = $NorisOtrsReport::OTRS_API::TicketObject->TicketSearch(
@@ -290,12 +289,12 @@ sub generate {
 
         # fetch data, run conversions, store result in Processors
         for my $TicketID (@TicketIDs) {
-            my %Data = ();
+            my %Data = ();            
             for my $Collector (@$Collectors) {
-                $Collector->( $UserID, $Conversions, $TicketID, \%Data );
+                $Collector->($UserID, $Conversions, $TicketID, \%Data);
             }
             for my $Processor (@$Processors) {
-                $Processor->item( \%Data );
+                $Processor->item(\%Data);
             }
         }
         for my $Processor (@$Processors) {
@@ -304,7 +303,7 @@ sub generate {
     }
     
     for my $Processor (@$Processors) {
-        $Processor->finish();
+            $Processor->finish();
     }
     return 1;
 }
@@ -356,21 +355,17 @@ Parameter etc.: Siehe Abschnitt "Collectors".
 =cut
 
 sub data_otrs_ticket {
-    my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
-    croak "data_otrs_ticket should be the first collector, working on an empty hash"
-      if %$Data;
-    %$Data = $NorisOtrsReport::OTRS_API::TicketObject->TicketGet(
-        TicketID => $TicketID,
-        UserID   => $UserID
-    );
-    apply_conversions( $Conversions, $Data );
+    my ($UserID, $Conversions, $TicketID, $Data) = @_;
+    croak "data_otrs_ticket should be the first collector, working on an empty hash" if %$Data;
+    %$Data = $NorisOtrsReport::OTRS_API::TicketObject->TicketGet(TicketID => $TicketID, UserID => $UserID);
+    apply_conversions($Conversions, $Data);
     return 1;
 }
 
 =head4 data_customtext()
 
-Dieser Collector holt seine Daten nicht direkt aus otrs, sondern aus
-C<%$Data>, soweit es von data_otrs_ticket() befüllt wurde. Daher
+Dieser Collector holt seine Daten nicht direkt aus otrs holt, sondern aus
+C<%$Data>, soweit es von data_otrs_ticket() befüllt worde. Daher
 muß unbedingt data_otrs_ticket() in der Liste der Collectors
 vorher stehen!
 
@@ -396,33 +391,32 @@ Für das Freitextfeld "Mandant", das in otrs im Freitextfeld Nummer 8 abgelegt
 wird, wird also erzeugt:
             
     $Data->{customtext}->{Mandant} = {
-        id    => 8;
+        id => 8;
         value => 'Der Mandant';
     };
 
 =begin testing
 
 my $Data = { 
-    TicketFreeKey8  => 'Mandant',
+    TicketFreeKey8 => 'Mandant',
     TicketFreeText8 => 'Der Mandant'
 };
 
-data_customtext( 1, [], 10001234, $Data );
+data_customtext(1, [], 10001234, $Data);
 is_deeply $Data->{customtext},
-    {
-        Mandant => {
-            id    => 8,
-            value => 'Der Mandant',
+    { Mandant => 
+        { id => 8, 
+          value => 'Der Mandant',
         }
     };
 
-$Data = {
-    TicketFreeKey8  => 'MandantXXX',
+$Data = { 
+    TicketFreeKey8 => 'MandantXXX',
     TicketFreeText8 => 'Der Mandant'
 };
 is_deeply $Data->{customtext}, undef;
 
-data_customtext( 1, [], 10001234, $Data );
+data_customtext(1, [], 10001234, $Data);
 
 =end testing
 
@@ -447,57 +441,12 @@ sub data_customtext {
     return 1;
 }
 
-=head4 data_workedtime()
-
-Holt die gebuchte Arbeitszeit des Tickets.
-
-Die Funktion ergänzt C<%$Data> um Einträge, die die gebuchete
-Arbeitszeit (in Sekunden) des Tickets beinhalten.
-
-Geliefert werden:
-
-=over
-
-=item $Data->{workedtime}->{simple_sum}
-
-Die gebuchte Zeit auf das Ticket mit Faktoren
-
-=item $Data->{workedtime}->{weighted_sum}
-
-und ohne Faktoren.
-
-=back
-
-Für das Freizeit/datumfeld "Incident-Resolved", das in otrs im Freitextfeld Nummer 3 abgelegt 
-wird, wird also erzeugt:
-            
-    $Data->{customdate}->{''} = {
-        id => 3,
-    };
-
-=begin testing
-
-my $Data = { };
-data_workedtime( 1, [], 10001234, $Data );
-is_deeply $Data->{workedtime}, { simple_sum => 0, weighted_sum => 0 } ;
-    
-=end testing
-
-=cut
-
-sub data_workedtime {
-    my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
-    my $WorkedTime = $NorisOtrsReport::OTRS_API::TicketObject->TicketWorkedTimeGet( TicketID => $TicketID );
-    $Data->{workedtime} = $WorkedTime;
-    return 1;
-}
-
 =head4 data_customdate()
 
 Vergleichbar mit data_customtext(), nur eben für TicketFreeTime ...
 
-Dieser Collector holt seine Daten nicht direkt aus otrs, sondern aus
-C<%$Data>, soweit es von data_otrs_ticket() befüllt wurde. Daher
+Dieser Collector holt seine Daten nicht direkt aus otrs holt, sondern aus
+C<%$Data>, soweit es von data_otrs_ticket() befüllt worde. Daher
 muß unbedingt data_otrs_ticket() in der Liste der Collectors
 vorher stehen!
 
@@ -530,38 +479,36 @@ wird, wird also erzeugt:
 
 =begin testing
 
-my $Data = { TicketFreeTime3 => '2009-04-01 13:31:42', };
-data_customdate( 1, [], 10001234, $Data );
+my $Data = { 
+    TicketFreeTime3 => '2009-04-01 13:31:42',
+};
+
+data_customdate(1, [], 10001234, $Data);
 is_deeply $Data->{customdate},
-  {
-    'Incident-Resolved' => {
-        id   => 3,
-        date => {
-            year   => 2009, month  => "04", day    => "01",
-            hour   => 13,   minute => 31,   second => 42,
-            epoch => convert_isodate_to_date_hash('2009-04-01 13:31:42')->{epoch}
+    { 'Incident-Resolved' => 
+        { id => 3, 
+          date => { year => 2009, month => "04", day => "01",
+                    hour => 13, minute => 31, second => 42,
+                    epoch => convert_isodate_to_date_hash('2009-04-01 13:31:42')->{epoch}
+                  }
         }
-    }
-  };
+    };
     
 =end testing
 
 =cut
 
 sub data_customdate {
-    my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
+    my ($UserID, $Conversions, $TicketID, $Data) = @_;
     my %Customdate = ();
-    for my $Key ( keys %TICKET_FREE_DATE_KEYS ) {
+    for my $Key (keys %TICKET_FREE_DATE_KEYS) {
         my $Index = $TICKET_FREE_DATE_KEYS{$Key};
-        if ( $Data->{"TicketFreeTime$Index"} ) {
-            $Customdate{$Key} = {
-                id   => $Index,
-                date => convert_isodate_to_date_hash(
-                    $Data->{"TicketFreeTime$Index"}
-                )
-            };
+        if ($Data->{"TicketFreeTime$Index"}) {
+            $Customdate{$Key} = { 
+                id => $Index,
+                date => convert_isodate_to_date_hash($Data->{"TicketFreeTime$Index"}) };
         }
-        apply_conversions( $Conversions, \%Customdate );
+        apply_conversions($Conversions, \%Customdate);
     }
     $Data->{customdate} = \%Customdate;
     return 1;
@@ -599,18 +546,18 @@ sub make_data_articles {
     my ($ArticleTypes) = @_;
     
     return sub {
-        my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
+        my ($UserID, $Conversions, $TicketID, $Data) = @_;
         my @Articles = $NorisOtrsReport::OTRS_API::TicketObject->ArticleGet(
-            TicketID    => $TicketID,
-            UserID      => $UserID,
-            ArticleType => $ArticleTypes,
-        );
+                                TicketID => $TicketID,
+                                UserID => $UserID,
+                                ArticleType => $ArticleTypes,
+                            );
         for my $Item (@Articles) {
-            apply_conversions( $Conversions, $Item );
+            apply_conversions($Conversions, $Item);
         }
         $Data->{articles} = \@Articles;
         return 1;
-    };
+    }
 }
 
 =head4 data_state_history()
@@ -629,12 +576,11 @@ Es werden für pro Status-Historie-Eintrag folgende Daten erzeugt:
 =cut
 
 sub data_state_history {
-    my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
-
+    my ($UserID, $Conversions, $TicketID, $Data) = @_;
     my @History = $NorisOtrsReport::OTRS_API::TicketObject->HistoryGet(
-        TicketID => $TicketID,
-        UserID   => $UserID,
-    );
+                        TicketID => $TicketID,
+                        UserID => $UserID,
+                      );
     for my $Item (@History) {
         delete $Item->{UserPw};
         apply_conversions($Conversions, $Item);
@@ -648,7 +594,7 @@ sub data_state_history {
     $Data->{history} = \@History;
 }
 
-=head4 data_linked_tickets()
+=head4 data_state_history()
 
 Ein Collector, der die verlinkten Tickets herunterlädt.
 Zum Interface siehe die Beschreibung zu Collectors.
@@ -710,18 +656,16 @@ Dieses Ticket (das in $TicketID) ist das untergeordnete.
 =cut
 
 sub data_linked_tickets {
-    my ( $UserID, $Conversions, $TicketID, $Data ) = @_;
-
+    my ($UserID, $Conversions, $TicketID, $Data) = @_;
     my $LinkList = $NorisOtrsReport::OTRS_API::LinkObject->LinkListWithData(
-        Object  => 'Ticket',
-        Key     => $TicketID,
-        Object2 => 'Ticket',
-        State   => 'Valid',
-        UserID  => $UserID,
-    );
+            Object    => 'Ticket',
+            Key       => $TicketID,
+            Object2   => 'Ticket',
+            State     => 'Valid',
+            UserID    => $UserID,
+        );
     
-    $Data->{'linked-tickets'} = $LinkList->{Ticket}
-      if exists $LinkList->{Ticket};
+    $Data->{'linked-tickets'} = $LinkList->{Ticket} if exists $LinkList->{Ticket};
     # LinkTyp -> Source|Target->{ id=>1}
 }
     
@@ -752,8 +696,8 @@ sub load_fixture {
     my ($Filename) = @_;
 
     my $fh;
-    open( $fh, "<:utf8", $Filename ) || die "Cannot open $Filename";
-    my $FixtureString = join( "", <$fh> );
+    open ($fh, "<:utf8", $Filename) || die "Cannot open $Filename";
+    my $FixtureString = join("", <$fh>);
     close $fh;
     my $Result = eval($FixtureString);
     die "Cannot load $Filename, eval complaines: $@" if $@;
@@ -763,26 +707,26 @@ sub load_fixture {
 }
 
 sub run_fixture_conversions {
-    my ( $Conversions, $Data ) = @_;
-    apply_conversions( $Conversions, $Data );
-    if ( ref $Data->{articles} ) {
-        for my $article ( @{ $Data->{articles} } ) {
-            apply_conversions( $Conversions, $article );
+    my ($Conversions, $Data) = @_;
+    apply_conversions($Conversions, $Data);
+    if (ref $Data->{articles}) {
+        for my $article (@{$Data->{articles}}) {
+            apply_conversions($Conversions, $article);
         }
     }
-    if ( ref $Data->{history} ) {
-        for my $item ( @{ $Data->{history} } ) {
-            apply_conversions( $Conversions, $item );
+    if (ref $Data->{history}) {
+        for my $item (@{$Data->{history}}) {
+            apply_conversions($Conversions, $item);
         }
     }
-    if ( defined $Data->{customtext} ) {
-        for my $d ( values %{ $Data->{customtext} } ) {
-            apply_conversions( $Conversions, $d );
+    if (defined $Data->{customtext}) {
+        for my $d (values %{$Data->{customtext}}) {
+            apply_conversions($Conversions, $d);
         }
     }
-    if ( defined $Data->{customdate} ) {
-        for my $d ( values %{ $Data->{customtext} } ) {
-            apply_conversions( $Conversions, $d );
+    if (defined $Data->{customdate}) {
+        for my $d (values %{$Data->{customtext}}) {
+            apply_conversions($Conversions, $d);
         }
     }
     return $Data;
@@ -801,13 +745,13 @@ Wert als Parameter und geben eine Hash-Referenz mit dem Ergebnis zurück.
 
 my $Data = { bla => '2009-04-01 13:31:42' };
 my $Result = NorisOtrsReport::Framework::apply_conversions(
-    [ [ 'bla', \&convert_isodate_to_date_hash, 'foo' ] ], $Data );
+    [[ 'bla', \&convert_isodate_to_date_hash, 'foo']], 
+    $Data);
 is_deeply $Data->{foo},
-  {
-    year   => 2009, month  => "04", day    => "01",
-    hour   => 13,   minute => 31,   second => 42,
-    epoch  => convert_isodate_to_date_hash('2009-04-01 13:31:42')->{epoch}
-  };
+        { year => 2009, month => "04", day => "01",
+          hour => 13, minute => 31, second => 42,
+          epoch => convert_isodate_to_date_hash('2009-04-01 13:31:42')->{epoch}
+        };
 is $Data->{bla}, '2009-04-01 13:31:42';
 
 =end testing
@@ -815,11 +759,11 @@ is $Data->{bla}, '2009-04-01 13:31:42';
 =cut
 
 sub apply_conversions {
-    my ( $Conversions, $Data ) = @_;
-    my ( $Key, $Fn );
+    my ($Conversions, $Data) = @_;
+    my ($Key, $Fn);
     for my $Item (@$Conversions) {
-        my ( $Key, $Fn, $Target ) = @$Item;
-        $Data->{$Target} = $Fn->( $Data->{$Key} ) if exists $Data->{$Key};
+        my ($Key, $Fn, $Target) = @$Item;
+        $Data->{$Target} = $Fn->($Data->{$Key}) if exists $Data->{$Key};
     }
 }
 
@@ -835,17 +779,16 @@ zurück.
 
 =begin testing
 
-use NorisOtrsReport::Framework qw(:convert);
-my $date = convert_epoch_to_date_hash(1233836217);
-is_deeply $date,
-  {
-    year   => "2009" month  => "2",  day    => "5",
-    hour   => "13",  minute => "16", second => "57",
-    epoch  => "1233836217",
-  };
+ use NorisOtrsReport::Framework qw(:convert);
+ my $date = convert_epoch_to_date_hash(1233836217);
+ is_deeply $date,
+    { day=>"5", epoch=>"1233836217", hour=>"13",
+      minute=>"16", month=>"2", second=>"57", year=>"2009" 
+    };
 
 =end testing
 
+
 =cut
 
 # Cachen der Zeitzone zur Optimierung der Performance (siehe #10012664)
@@ -856,14 +799,14 @@ sub convert_epoch_to_date_hash {
     return unless defined $epoch;
     my $date = DateTime->from_epoch( epoch => $epoch, time_zone => $tz_local );
     return {
-        epoch  => $epoch,
-        year   => $date->year(),
-        month  => $date->month(),
-        day    => $date->day(),
-        hour   => $date->hour(),
+        epoch => $epoch,
+        day => $date->day(),
+        month => $date->month(),
+        year => $date->year(),
+        hour => $date->hour(),
         minute => $date->minute(),
         second => $date->second(),
-    };
+    }
 }
 
 =head4 convert_isodate_to_date_hash()
@@ -878,34 +821,33 @@ zurück.
 
 =begin testing
 
-use NorisOtrsReport::Framework qw(:convert);
-my $Result   = convert_isodate_to_date_hash('2009-02-05 13:16:57');
-my $Expected = {
-    year   => "2009" month  => "02", day    => "05",
-    hour   => "13",  minute => "16", second => "57",
-    epoch  => '1233836217',
-};
-is_deeply $Result, $Expected;
-
-# Und die Zeitzonen stimmen auch noch zwischen den beiden Funktionen überein:
-is convert_epoch_to_date_hash( $Result->{epoch} )->{hour}, $Expected->{hour};
+ use NorisOtrsReport::Framework qw(:convert);
+ my $Result = convert_isodate_to_date_hash('2009-02-05 13:16:57');
+ my $Expected = { day=>"05", epoch=>'1233836217', hour=>"13",
+      minute=>"16", month=>"02", second=>"57", year=>"2009" 
+    };
+ is_deeply $Result, $Expected;
+ # Und die Zeitzonen stimmen auch noch zwischen den beiden Funktionen überein:
+ is convert_epoch_to_date_hash($Result->{epoch})->{hour}, $Expected->{hour};
+    
 
 =end testing
 
 =cut
 
+
 sub convert_isodate_to_date_hash {
     my ($isodate) = @_;
     return {} unless $isodate =~ /^(....)-(..)-(..) (..):(..)(?::(..))?$/;
     my %result = (
-        year   => $1,
-        month  => $2,
-        day    => $3,
-        hour   => $4,
+        year => $1,
+        month => $2,
+        day => $3,
+        hour => $4,
         minute => $5,
         second => $6 || 0,
     );
-    my $date = DateTime->new( %result, time_zone => $tz_local );
+    my $date = DateTime->new(%result, time_zone => $tz_local);
 
     $result{epoch} = $date->epoch();
     return \%result;
