@@ -1,9 +1,9 @@
 package NorisOtrsReport::Processor;
 
 use Moose;
-use NorisOtrsReport::Renderer::JSON;
+use NorisOtrsReport::JSONRenderer;
 
-has 'Renderer' => (is => 'ro', default => sub { NorisOtrsReport::Renderer::JSON->new() }, isa => 'NorisOtrsReport::Renderer');
+has 'Renderer' => (is => 'ro', default => sub { NorisOtrsReport::JSONRenderer->new() }, isa => 'NorisOtrsReport::Renderer');
 
 sub start {
     my ($Self) = @_;
